"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSvelteVersionBucket = exports.getKitVersionBucket = void 0;
const semver_1 = require("semver");
function getKitVersionBucket(version) {
    if (!version) {
        return 'none';
    }
    const minVer = (0, semver_1.minVersion)(version);
    if (!minVer) {
        return 'invalid';
    }
    if ((0, semver_1.lt)(minVer, '1.0.0')) {
        return '0.x';
    }
    else if ((0, semver_1.lt)(minVer, '1.24.0')) {
        return '>=1.0.0 <1.24.0';
    }
    else {
        // This is the version when the client-side invalidation fix was released
        // https://github.com/sveltejs/kit/releases/tag/%40sveltejs%2Fkit%401.24.0
        // https://github.com/sveltejs/kit/pull/10576
        return '>=1.24.0';
    }
}
exports.getKitVersionBucket = getKitVersionBucket;
function getSvelteVersionBucket(version) {
    if (!version) {
        return 'none';
    }
    const minVer = (0, semver_1.minVersion)(version);
    if (!minVer) {
        return 'invalid';
    }
    if ((0, semver_1.lt)(minVer, '3.0.0')) {
        return '<3.0.0';
    }
    if ((0, semver_1.lt)(minVer, '4.0.0')) {
        return '3.x';
    }
    if ((0, semver_1.lt)(minVer, '5.0.0')) {
        return '4.x';
    }
    // Svelte 5 isn't released yet but it's being worked on
    return '>4.x';
}
exports.getSvelteVersionBucket = getSvelteVersionBucket;
//# sourceMappingURL=utils.js.map