"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackageVersion = exports.hasPackageInstalled = exports.findInstalledPackageFromList = void 0;
/**
 * Checks if @param packageJson has any of the @param packageNamesList package names
 * listed as a dependency or devDependency.
 * If so, it returns the first package name that is found, including the
 * version (range) specified in the package.json.
 */
function findInstalledPackageFromList(packageNamesList, packageJson) {
    return packageNamesList
        .map((packageName) => ({
        name: packageName,
        version: getPackageVersion(packageName, packageJson),
    }))
        .find((sdkPackage) => !!sdkPackage.version);
}
exports.findInstalledPackageFromList = findInstalledPackageFromList;
function hasPackageInstalled(packageName, packageJson) {
    return getPackageVersion(packageName, packageJson) !== undefined;
}
exports.hasPackageInstalled = hasPackageInstalled;
function getPackageVersion(packageName, packageJson) {
    return (packageJson?.dependencies?.[packageName] ||
        packageJson?.devDependencies?.[packageName]);
}
exports.getPackageVersion = getPackageVersion;
//# sourceMappingURL=package-json.js.map