"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const package_manager_1 = require("../../src/utils/package-manager");
(0, vitest_1.describe)('_detectPackageManager', () => {
    (0, vitest_1.it)('returns the detected package manager if exactly one is found', () => {
        const pnpm = { ...package_manager_1.PNPM, detect: () => true };
        const packageManager = (0, package_manager_1._detectPackageManger)([
            { ...package_manager_1.NPM, detect: () => false },
            { ...package_manager_1.YARN_V1, detect: () => false },
            pnpm,
        ]);
        (0, vitest_1.expect)(packageManager).toBe(pnpm);
    });
    (0, vitest_1.it)('returns null if no package manager is found', () => {
        const packageManager = (0, package_manager_1._detectPackageManger)([
            { ...package_manager_1.NPM, detect: () => false },
            { ...package_manager_1.YARN_V1, detect: () => false },
        ]);
        (0, vitest_1.expect)(packageManager).toBeNull();
    });
    (0, vitest_1.it)('returns null if multiple package managers are found', () => {
        const packageManager = (0, package_manager_1._detectPackageManger)([
            { ...package_manager_1.NPM, detect: () => true },
            { ...package_manager_1.YARN_V1, detect: () => true },
        ]);
        (0, vitest_1.expect)(packageManager).toBeNull();
    });
});
//# sourceMappingURL=package-manager.test.js.map